;
; Cursor control routines for the HD44780 (and compatible) display controller
;
; LCD display size: 4x20
;
; Andreas Tnne
; atoenne@t-online.de
;
; No copyright. This is free software. Thank you for naming the author
;
; Web released version 1.0
;
;***********************************************************************

;*********************   USAGE   ***************************************
;
; .include "LCD Driver.asm"
;
; .SET Scratch				Garbage register that is modified arbitrarily
; .SET LCDCursorTmp			Additional register (> 15). Is not modified!
;
; rcall LCD420SetCursorPos		Move the cursor to the position LCDValue
;								In: LCDValue, Modifies: 

; NOTE: these are a subset of my LCD cursor control routines.
; see www.atoenne.de for a download of the full file
; the full set of routines simply does not fit in the 2313 and hence I cut out what I did not need.
; actually not very much is left :-)


;###################################################################
;
; Routines for cursor control
; WARNING: these routines do not work properly if the display is shifted since there
; 			is no way to read the current shift status and we depend on the current address
;			to determine where we are in logical display coordinates. 
;
; Each kind of display (number of lines, chars per line) has a different coordinate system
; for addressing cursor positions.
; These cursor control routines implement a logical coordinate system that is not necessarily the
; same as the physical cursor model of the HD44780. For example, the cursor move right command of the
; LCD advances through the display memory, which is interleaved on many multi-line displays.


; **************** 4 x 20 *******************
;
; Think of a 4x20 display as a 2x40 display that has been split to half and interleaved.
; I.e. the second half of line one is put after the first half of line two.
; Each of the two physical lines is 0x28 long. Address 0x28-0x3F is unused. 
; Hence Bit6 of the address indicates safely line 2 or 4. 

.SET LCD420LineLength=0x14

.SET LCD420AddressLine1=0x00
.SET LCD420AddressLine2=0x40
.SET LCD420AddressLine3=0x14
.SET LCD420AddressLine4=0x54

; the first not reachable address is 0x68 = 0b01101000


;******* LCD420SetCursorPos *******
; Move the cursor to the position given in LCDValue for YYXXXXXX (Y is 0-3, X is 0-39)


LCD420SetCursorPos:
	push LCDCursorTmp
	ldi LCDCursorTmp, LCD420LineLength
	mov Scratch, LCDValue
	andi LCDValue, 0b00111111			; retain the X value
	sbrc Scratch, 7						; skip if in lines 0 or 1
	add LCDValue, LCDCursorTmp			; adjust line interleaving for lines 2 or 3
LCD420SetCursorPos1:
	ldi LCDCursorTmp, 0x40
	sbrc Scratch, 6						; skip if in line 0 or 2 (lower bit of Y clear)
	add LCDValue, LCDCursorTmp			; in line 1 or 3 add the necessary offset 0x40 on the column address
	LCDSetDisplayAddress
	pop LCDCursorTmp
	ret
