;
; Text routines for the HD44780 (and compatible) display controller
;
; Andreas Tnne
; atoenne@t-online.de
;
; No copyright. This is free software. Thank you for naming the author
;
; Web released version 1.0
;
;***********************************************************************

;*********************   USAGE   ***************************************
;
; .include "LCD Driver.asm"
; .include "LCDCursorXXXX.asm"		depending on your display size
;
;
; LCDTextP					Write a text pointed to by register Z 
;							In: Z, LCDValue Modifies:  
; rcall LCDWriteHex			Write the value of LCDValue as a hex number
;							In: LCDValue Modifies: 
;
; Note: we assume that the predefined register Scratch is r0

;****** LCDTextPR ******
; Write a text in the program memory at address Z  of length LCDValue to the LCD
; Note: Z must be twice the memory address since bit 0 is used to split the addressed
; word to bytes

LCDTextPR:
	push r0
LCDTextPLoop:
	push LCDValue
	lpm							; stores the program memory byte in register r0
	mov LCDValue, r0
	rcall LCDWriteData
	adiw ZH:ZL, 1
	pop LCDValue
	dec LCDValue
	brne LCDTextPLoop
	pop r0
	ret

;******* LCDTextP Address Length **********

.MACRO LCDTextP
	ldi ZL, LOW(@0 * 2)
	ldi ZH, HIGH(@0 * 2)
	ldi LCDValue, @1
	rcall LCDTextPR
.ENDMACRO

;****** LCDWriteHex ******
; Write the value in LCDValue as a hex value to the LCD

LCDWriteHex:
	push LCDValue
	swap LCDValue
	andi LCDValue, 0x0F
	cpi LCDValue, 0xA
	brlt LCDHexA1					; < 10
	subi LCDValue, -7				; +7 = distance between "9" and "A"
LCDHexA1:
	subi LCDValue, -0x30			; +"0"
	rcall LCDWriteData
	pop LCDValue
	andi LCDValue, 0x0F
	cpi LCDValue, 0xA
	brlt LCDHexA2					; < 10
	subi LCDValue, -7				; +7 = distance between "9" and "A"
LCDHexA2:
	subi LCDValue, -0x30			; +"0"
	rcall LCDWriteData
	ret
